'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _capitalize2 = require('lodash/capitalize');

var _capitalize3 = _interopRequireDefault(_capitalize2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _loreUtils = require('lore-utils');

var _loreReactForms = require('lore-react-forms');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _createReactClass2.default)({
  displayName: 'Optimistic',

  propTypes: {
    data: _propTypes2.default.object,
    validators: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.object]),
    onChange: _propTypes2.default.func,
    onSubmit: _propTypes2.default.func,
    callbacks: _propTypes2.default.object,
    schema: _propTypes2.default.object.isRequired,
    fieldMap: _propTypes2.default.object.isRequired,
    actionMap: _propTypes2.default.object.isRequired,
    fields: _propTypes2.default.object.isRequired,
    actions: _propTypes2.default.array.isRequired
  },

  getInitialState: function getInitialState() {
    var data = this.props.data;


    return {
      data: data || {}
    };
  },

  request: function request(data) {
    var modelName = this.props.modelName;
    var request = this.props.request;

    return request ? request(data) : lore.actions[modelName].create(data).payload;
  },

  onSubmit: function onSubmit() {
    var data = this.state.data;

    this.request(data);
    this.onCancel();
  },

  onCancel: function onCancel(data) {
    this.props.onCancel(data);
  },

  onChange: function onChange(name, value) {
    var nextData = (0, _merge3.default)({}, this.state.data);
    nextData[name] = value;
    this.setState({
      data: nextData
    });
  },

  getValidators: function getValidators(data) {
    var validators = this.props.validators;

    return (0, _loreUtils.result)(validators || {}, data);
  },

  render: function render() {
    var modelName = this.props.modelName;
    var _props = this.props,
        _props$title = _props.title,
        title = _props$title === undefined ? 'Create ' + (0, _capitalize3.default)(modelName) : _props$title,
        _props$description = _props.description,
        description = _props$description === undefined ? '' : _props$description,
        schema = _props.schema,
        fieldMap = _props.fieldMap,
        actionMap = _props.actionMap,
        fields = _props.fields,
        actions = _props.actions;
    var data = this.state.data;

    var validators = this.getValidators(data);

    return _react2.default.createElement(
      'div',
      { className: 'modal-dialog' },
      _react2.default.createElement(
        'div',
        { className: 'modal-content' },
        _react2.default.createElement(
          'div',
          { className: 'modal-header' },
          _react2.default.createElement(
            'button',
            { type: 'button', className: 'close', onClick: this.onCancel },
            _react2.default.createElement(
              'span',
              null,
              '\xD7'
            )
          ),
          title ? _react2.default.createElement(
            'h4',
            { className: 'modal-title' },
            title
          ) : null,
          description ? _react2.default.createElement(
            'p',
            { className: 'help-block' },
            description
          ) : null
        ),
        _react2.default.createElement(_loreReactForms.SchemaForm, {
          data: data,
          validators: validators,
          onChange: this.onChange,
          callbacks: {
            onSubmit: this.onSubmit,
            onCancel: this.onCancel
          },
          schema: schema,
          fieldMap: fieldMap,
          actionMap: actionMap,
          fields: fields || {},
          actions: actions || []
        })
      )
    );
  }

});